package nl.iobyte.themeparkconnector.api.load.objects;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.config.objects.Configuration;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.load.interfaces.IDataLoader;
import nl.iobyte.themeparkconnector.api.operator.OperatorService;
import nl.iobyte.themeparkconnector.api.operator.objects.AttractionOperator;
import nl.iobyte.themeparkconnector.api.operator.objects.panel.OperatorItem;
import nl.iobyte.themeparkconnector.api.operator.objects.panel.OperatorItemState;
import nl.iobyte.themeparkconnector.api.operator.objects.panel.OperatorPanel;
import nl.iobyte.themeparkconnector.logger.ThemeParkConnectorLogger;
import org.bukkit.configuration.ConfigurationSection;
import java.io.File;
import java.util.LinkedHashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class OperatorDataLoader implements IDataLoader {

    /**
     * Called when loading data
     */
    public void load() {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();

        File folder = new File(ThemeParkConnector.getInstance().getDataFolder(), "panels");
        if(!folder.exists()) {
            if (!folder.mkdirs()) {
                ThemeParkConnectorLogger.toConsole("Unable to create 'panels' folder");
                return;
            }
        }

        Set<String> keys = new HashSet<>(api.getAttractionService().getRegions().keySet());
        if(!new File(folder, "templates").exists()) {
            keys.add("templates");
        } else {
            loadPanels(api, "panels/templates");
        }

        File[] files = folder.listFiles();
        if(files != null && files.length != 0)  {
            for (File file : files) {
                String name = file.getName();
                if (name.equals("templates"))
                    continue;

                if (file.isFile()) {
                    ThemeParkConnectorLogger.toConsole("File '" + name + "' in 'panels' folder not allowed");
                    if (!file.delete())
                        ThemeParkConnectorLogger.toConsole("Unable to delete file '" + name + "' in 'panels' folder");

                    continue;
                }

                if (!api.getAttractionService().hasRegion(name)) {
                    ThemeParkConnectorLogger.toConsole("Folder '" + name + "' in 'panels' folder not allowed");
                    if (!file.delete())
                        ThemeParkConnectorLogger.toConsole("Unable to delete folder '" + name + "' in 'panels' folder");

                    continue;
                }

                keys.remove(name);
                loadPanels(api, "panels/"+name);
            }
        }

        for(String name : keys) {
            if (!new File(ThemeParkConnector.getInstance().getDataFolder(), "panels/" + name).mkdir()) {
                ThemeParkConnectorLogger.toConsole("Unable to create folder '" + name + "' in 'panels' folder");
                continue;
            }

            loadPanels(api, "panels/"+name);
        }

        for(String id : ThemeParkConnector.getInstance().getAPI().getOperatorService().getPanels().keySet()) {
            if(api.getAttractionService().hasAttraction(id))
                continue;

            ThemeParkConnector.getInstance().getAPI().getOperatorService().removePanel(id);
        }
    }

    private void loadPanels(ThemeParkAPI api, String path) {
        File folder = new File(ThemeParkConnector.getInstance().getDataFolder(), path);
        File[] files = folder.listFiles();

        Set<String> keys;
        if(folder.getName().equals("templates")) {
            keys = new HashSet<>();
        } else {
            keys = new HashSet<>(api.getAttractionService().getRegion(folder.getName()).getAttractions().keySet());
        }

        if(files != null && files.length != 0) {
            for (File file : files) {
                String name = file.getName();
                if (!file.isFile()) {
                    ThemeParkConnectorLogger.toConsole("Folder '" + name + "' in '" + path + "' folder not allowed");
                    if (!file.delete())
                        ThemeParkConnectorLogger.toConsole("Unable to delete folder '" + name + "' in 'panels' folder");

                    continue;
                }

                if (!name.endsWith(".yml") && !name.endsWith(".json")) {
                    ThemeParkConnectorLogger.toConsole("Extension of file '" + name + "' in '" + path + "' folder not allowed");
                    continue;
                }

                keys.remove(name.replaceAll("(.json|.yml)", ""));
                loadPanel(api, path, name);
            }
        }

        for(String name : keys)
            loadPanel(api, path, name+".yml");
    }

    private void loadPanel(ThemeParkAPI api, String path, String id) {
        Configuration configuration = new Configuration(ThemeParkConnector.getInstance(), path+"/"+id, false);
        id = id.replaceAll("(.json|.yml)", "");
        if(configuration.contains("template")) {
            loadTemplatedPanel(ThemeParkConnector.getInstance().getAPI().getOperatorService(), configuration, id);
            if(!api.getAttractionService().hasAttraction(id) || !ThemeParkConnector.getInstance().getAPI().getOperatorService().hasPanel(id))
                return;

            String permission = configuration.getString("permission");
            String start_command = configuration.getString("start_command");
            String stop_command = configuration.getString("stop_command");
            ThemeParkConnector.getInstance().getAPI().getOperatorService().addOperator(new AttractionOperator(
                    id,
                    permission,
                    start_command,
                    stop_command
            ));
            return;
        }

        if(!configuration.contains("items"))
            return;

        ConfigurationSection items = configuration.getSection("items");
        if(items == null || items.getKeys(false).isEmpty())
            return;

        ConfigurationSection states;
        OperatorPanel panel = new OperatorPanel(id);
        for(String item_id : items.getKeys(false)) {
            path = "items."+item_id;
            String name = configuration.getString(path+".name");
            String active_state = configuration.getString(path+".active_state");
            if(!configuration.contains(path+".states"))
                continue;

            states = configuration.getSection(path+".states");
            if(states == null || states.getKeys(false).isEmpty())
               continue;

            OperatorItem item = new OperatorItem(item_id, name, active_state);
            for(String state_id : states.getKeys(false)) {
                path = "items." + item_id + ".states." + state_id;
                name = configuration.getString(path+".name");
                String cover = configuration.getString(path+".cover");
                String command = configuration.getString(path+".command");

                String text_color = configuration.getString(path+".text_color");
                String background_color = configuration.getString(path+".background_color");
                if(text_color != null && !text_color.isEmpty() && background_color != null && !background_color.isEmpty()) {
                    boolean glow = configuration.getBoolean(path+".glow");

                    item.addState(new OperatorItemState(state_id, name, cover, command, text_color, background_color, glow));
                    continue;
                }

                item.addState(new OperatorItemState(state_id, name, cover, command));
            }

            panel.addItem(item);
        }

        if(panel.getItems().size() == 0)
            return;

        ThemeParkConnector.getInstance().getAPI().getOperatorService().addPanel(panel);
        if(!api.getAttractionService().hasAttraction(id))
            return;

        String permission = configuration.getString("permission");
        String start_command = configuration.getString("start_command");
        String stop_command = configuration.getString("stop_command");
        ThemeParkConnector.getInstance().getAPI().getOperatorService().addOperator(new AttractionOperator(
                id,
                permission,
                start_command,
                stop_command
        ));
    }

    private void loadTemplatedPanel(OperatorService service, Configuration configuration, String id) {
        OperatorPanel panel = service.getPanel(configuration.getString("template"));
        if(panel == null)
            return;

        if(!configuration.contains("items"))
            return;

        ConfigurationSection items = configuration.getSection("items");
        if(items == null || items.getKeys(false).isEmpty())
            return;

        String path;
        Map<String, String> map;
        ConfigurationSection states;
        Map<String, Map<String, String>> commandMap = new LinkedHashMap<>();
        for(String item_id : items.getKeys(false)) {
            path = "items."+item_id;
            if(!configuration.contains(path+".states"))
                continue;

            states = configuration.getSection("states");
            if(states == null || states.getKeys(false).isEmpty())
                continue;

            map = new LinkedHashMap<>();
            for(String state_id : items.getKeys(false)) {
                path = "items." + item_id + ".states." + state_id;
                String command = configuration.getString(path+".command");

                map.put(state_id, command);
            }

            if(map.size() == 0)
                continue;

            commandMap.put(item_id, map);
        }

        if(commandMap.size() == 0)
            return;

        service.addPanel(panel.copy(id, commandMap));
    }

}
