package nl.iobyte.themeparkconnector.api.operator.objects.panel;

import nl.iobyte.themepark.scheduler.ThemeParkScheduler;
import org.bukkit.Bukkit;
import java.util.HashMap;
import java.util.Map;

public class OperatorItem {

    private final String id, name;
    private final Map<String, OperatorItemState> states;
    private String active_state;

    public OperatorItem(String id, String name, String active_state) {
        this.id = id;
        this.name = name;
        this.active_state = active_state;
        this.states = new HashMap<>();
    }

    public OperatorItem(String id, String name, String active_state, Map<String, OperatorItemState> states) {
        this.id = id;
        this.name = name;
        this.active_state = active_state;
        this.states = states;
    }

    public String getID() {
        return id;
    }

    public String getName() {
        return name;
    }

    public OperatorItemState getActiveState() {
        return getState(active_state);
    }

    public void setActiveState(String active_state) {
        this.active_state = active_state;
    }

    public void addState(OperatorItemState state) {
        if(hasState(state.getID()))
            return;

        states.put(state.getID(), state);
    }

    public boolean hasState(String id) {
        return states.containsKey(id);
    }

    public OperatorItemState getState(String id) {
        return states.get(id);
    }

    public void click() {
        if(!states.containsKey(active_state))
            return;

        String cmd = getActiveState().getCommand();
        if(cmd == null || cmd.isEmpty())
            return;

        ThemeParkScheduler.runSync(() -> Bukkit.dispatchCommand(Bukkit.getConsoleSender(), cmd));
    }

    public OperatorItem copy(Map<String, String> commands) {
        Map<String, OperatorItemState> states = new HashMap<>();
        OperatorItemState state;
        for(Map.Entry<String, OperatorItemState> entrySet : this.states.entrySet()) {
            state = entrySet.getValue();
            states.put(
                    entrySet.getKey(),
                    state.copy(commands.containsKey(entrySet.getKey()) ? commands.get(entrySet.getKey()) : state.getCommand())
            );
        }

        return new OperatorItem(id, name, active_state, states);
    }

}
