package nl.iobyte.themeparkconnector.api.packet.handlers;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.ThemeParkConnectorAPI;
import nl.iobyte.themeparkconnector.api.message.Text;
import nl.iobyte.themeparkconnector.api.network.objects.INetworkingEvent;
import nl.iobyte.themeparkconnector.api.client.ClientConnection;
import nl.iobyte.themeparkconnector.api.network.packet.operator.OperatorRequestReplyPacket;
import nl.iobyte.themeparkconnector.api.operator.enums.OperatorRequestReply;
import nl.iobyte.themeparkconnector.api.operator.objects.AttractionOperator;
import nl.iobyte.themeparkconnector.api.packet.objects.PayloadHandler;
import nl.iobyte.themeparkconnector.api.packet.payloads.ClientConnectPayload;
import java.util.UUID;

public class ClientConnectHandler extends PayloadHandler<ClientConnectPayload> {

    /**
     * Player established connection with ThemeParkConnector
     * @param payload ClientConnectPayload
     */
    public void onReceive(ClientConnectPayload payload) {
        ThemeParkConnectorAPI api = ThemeParkConnector.getInstance().getAPI();
        ClientConnection connection = api.getNetworkingService().getClient(payload.getUUID());
        if(connection == null) {
            denyRequest(
                    api,
                    payload.getUUID(),
                    "No client connection found for uuid"
            );
            return;
        }

        connection.setUsedToken(payload.getToken());
        if(!api.getAuthenticationService().matchToken(payload.getUUID(), payload.getToken(), payload.getAttractionID())) {
            denyRequest(
                    api,
                    payload.getUUID(),
                    "Invalid token used to connect"
            );
            return;
        }

        if(api.getOperatorService().isOperating(payload.getUUID())) {
            denyRequest(
                    api,
                    payload.getUUID(),
                    "You're already operating an attraction"
            );
            return;
        }

        Attraction attraction = ThemePark.getInstance().getAPI().getAttractionService().getAttraction(payload.getAttractionID());
        if(attraction == null) {
            denyRequest(
                    api,
                    payload.getUUID(),
                    "Unable to find requested attraction"
            );
            return;
        }

        AttractionOperator operator = api.getOperatorService().getOperator(payload.getAttractionID());
        if(operator == null) {
            denyRequest(
                    api,
                    payload.getUUID(),
                    "No operator found for attraction: " + Text.strip(attraction.getName())
            );
            return;
        }


        if(!operator.canClientConnect(connection)) {
            denyRequest(
                    api,
                    payload.getUUID(),
                    "You're not allowed to operate attraction: " + Text.strip(attraction.getName())
            );
            return;
        }

        api.getOperatorService().startOperating(payload.getUUID(), payload.getAttractionID());
        api.getNetworkingService().send(
                api.getNetworkingService().getClient(payload.getUUID()),
                new OperatorRequestReplyPacket(
                        OperatorRequestReply.ACCEPT,
                        "You're now operating attraction: " + Text.strip(attraction.getName()),
                        operator.getPanel()
                )
        );

        connection.onConnect();

        for(INetworkingEvent event : ThemeParkConnector.getInstance().getAPI().getNetworkingService().getEvents())
            event.onClientOpen(connection);
    }

    private void denyRequest(ThemeParkConnectorAPI api, UUID uuid, String message) {
        api.getNetworkingService().send(
                api.getNetworkingService().getClient(uuid),
                new OperatorRequestReplyPacket(
                        OperatorRequestReply.DENY,
                        message
                )
        );
    }

}
