package nl.iobyte.themeparkconnector.api.state;

import nl.iobyte.themeparkconnector.api.state.objects.AbstractState;
import nl.iobyte.themeparkconnector.api.state.states.ConnectedState;
import nl.iobyte.themeparkconnector.api.state.states.ConnectingState;
import nl.iobyte.themeparkconnector.api.state.states.IdleState;
import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;

public class StateService {

    private AbstractState currentState = new IdleState();
    private final List<BiConsumer<AbstractState, AbstractState>> listeners = new ArrayList<>();

    /**
     * Register listener for State changes
     * @param listener BiConsumer<AbstractState, AbstractState>
     */
    public void addListener(BiConsumer<AbstractState, AbstractState> listener) {
        listeners.add(listener);
    }

    /**
     * Get current state
     * @return AbstractState
     */
    public AbstractState getCurrentState() {
        return currentState;
    }

    public boolean isConnecting() {
        return currentState instanceof ConnectingState;
    }

    public boolean isConnected() {
        return currentState instanceof ConnectedState;
    }

    /**
     * Set current state
     * @param currentState AbstractState
     */
    public void setState(AbstractState currentState) {
        AbstractState oldState = this.currentState;
        this.currentState = currentState;
        for(BiConsumer<AbstractState, AbstractState> listener : listeners)
            listener.accept(oldState, currentState);
    }

}
